/*
  * Copyright (C) 2006-2011 Nokia Corporation.
  *
  * This file is part of devicelock
  *
  * Authors: Brian McGillion <brian.mcgillion@symbio.com>,
  * 	     Denis Mingulov  <denis.mingulov@symbio.com>
  *
  */

/*!
  * \file devicelock.h
  * \author Brian McGillion <brian.mcgillion@symbio.com>,
  *         Denis Mingulov <denis.mingulov@symbio.com>
  * \brief Enumerations used both internally and externally
  */

#ifndef DEVICELOCK_H
#define DEVICELOCK_H

#ifndef QT_CORE_LIB
// Allow non Qt applications to use these defines.

//! The DBus system service provided by devicelock
#define DEVLOCK_SERVICE "com.nokia.devicelock"
//! The interface that the devicelock uses
#define DEVLOCK_INTERFACE "com.nokia.devicelock"
//! The DBus path of the service
#define DEVLOCK_PATH "/request"
//! Method used to determine the state of the devicelock
#define DEVLOCK_GET "getState"
//! Method used to set the state of the devicelock
#define DEVLOCK_SET "setState"
//! A DBus signal used to notify that the state of the lock has changed
#define DEVLOCK_SIGNAL "stateChanged"


//! \sa DeviceLock::DeviceLockEnums::LockType
typedef enum {
    TouchAndKeyboard = 0,   //!< TouchAndKeyboard - The touch screen lock
    Device                  //!< Device - The device lock, password protected lock screen
} LockType_t;

//! \sa DeviceLock::DeviceLockEnums::LockState
typedef enum
{
    Unlocked = 0,           //!< Unlocked - The lock is unlocked
    Locked,                 //!< Locked - The lock is being used
    Configuration,          //!< Configuration - Open the locks configuration settings
    WipeMMC,                //!< WipeMMC - Secure wipe of the device
    Inhibit,                //!< Inhibit - Stop the lock ui(s) from being displayed
    PasswordPrompt,         //!< PasswordPrompt - Display a password prompt UI
    Initializing,           //!< Initializing - the devicelock is in the process of initializing
    ChangePasswordPrompt,   //!< ChangePasswordPrompt - Display a password change prompt UI
    Undefined               //!< Undefined - The state of the lock is unknown or the lock does not exist
} LockState_t;

//! \sa DeviceLock::DeviceLockEnums::WipeType
typedef enum
{
    CleanDeviceWipe = 0,    //!< CleanDeviceWipe - clean device to 'Out-of-Box' state
    FullSecureWipe,         //!< FullSecureWipe - fully secure wipe, device has not be able to work at all
    UndefinedWipe           //!< UndefinedWipe - unknown wipe type
} WipeType_t;

#else // QT applications can make use of the rest of the enums.

//! The DBus system service provided by devicelock
#define DEVLOCK_SERVICE "com.nokia.devicelock"
//! The interface that the devicelock uses
#define DEVLOCK_INTERFACE "com.nokia.devicelock"
//! The DBus path of the service
#define DEVLOCK_PATH "/request"
//! Method used to determine the state of the devicelock
#define DEVLOCK_GET "getState"
//! Method used to set the state of the devicelock
#define DEVLOCK_SET "setState"
//! A DBus signal used to notify that the state of hte lock has changed
#define DEVLOCK_SIGNAL "stateChanged"

#include <QtCore/QMetaType>
#include <QtCore/QMap>
#include <QtCore/QVariant>
#include <QtCore/QPair>
#include <QtCore/QVariantHash>
#include <QtDBus/QDBusArgument>

namespace DeviceLock {

class DeviceLockEnums : public QObject
{
    Q_OBJECT
    Q_ENUMS(LockType);
    Q_ENUMS(LockState);

public:
    /*!
      * \enum LockType
      * \brief Enumeration of the valid locks for the device lock only.
      * \warning It is not used currently, the exact value does not matter now - an usual device lock is supported only now.
      */
    enum LockType
    {
        TouchAndKeyboard = 0,   /*!< TouchAndKeyboard - The touch screen lock. \deprecated It is deprecated. */
        Device                  /*!< Device - The device lock, password protected lock screen */
    };

    /*!
      * \enum LockState
      * \brief Enumeration of the valid states that a lock can be in
      */
    enum LockState
    {
        Unlocked = 0,           /*!< Unlocked - The lock is unlocked */
        Locked,                 /*!< Locked - The lock is being used */
        Configuration,          /*!< Configuration - Open the locks configuration settings */
        WipeMMC,                /*!< WipeMMC - Secure wipe of the device */
        Inhibit,                /*!< Inhibit - Stop the lock ui(s) from being displayed */
        PasswordPrompt,         /*!< PasswordPrompt - Display a password prompt UI */
        Initializing,           /*!< Initializing - the devicelock is in the process of initializing */
        ChangePasswordPrompt,   /*!< ChangePasswordPrompt - Display a password change prompt UI */
        CancelChainedPrompt,    /*!< CancelChainedPrompt - Cancel chained window (change) password prompts */
        Undefined               /*!< Undefined - The state of the lock is unknown or the lock does not exist */
    };

    /*!
      * \enum WipeType
      * \brief Enumeration of the valid wipe types. Has to be ordered from the 'simpliest' mode to the most security.
      */
    enum WipeType
    {
        CleanDeviceWipe = 0,    /*!< CleanDeviceWipe - clean device to 'Out-of-Box' state */
        FullSecureWipe,         /*!< FullSecureWipe - fully secure wipe, device has not be able to work at all */
        UndefinedWipe           /*!< UndefinedWipe - unknown wipe type */
    };

    /*!
      * \enum UiPageType
      * \brief Enumeration of the valid type of pages to create.
      */
    enum UiPageType
    {
        DeviceLock = 0,     /*!< DeviceLock - A Lock that will disable the screen requiring a password to unlock */
        PasscodePromptPage, /*!< PasscodePrompt - Request that the user enter a valid password */
        Settings,           /*!< Settings - A page that is created to hold the settings ui components */
        ChangePassword,     /*!< ChangePassword - A Page that is used to display the password change functionality */
        ForcePasswordChange,/*!< ForcePasswordChange - Similar to the ChangePassword page but the user can't choose cansel */
        CancelChainedWin    /*!< CancelChainedWin - Cancel chained window */
    };

    /*!
      * \enum ProvisioningSettings
      * \brief Enumeration of the valid keys for the provisioning settings' map
      */
    enum ProvisioningSettings
    {
        /*!
          * \brief Indicates whether a device password is required.
          *
          * This setting indicates whether a device password is required.
          * Value - Boolean. True if a device password is required, false otherwise.
          */
        DevicePasswordRequired = 0,

        // Password specific requirements
        /*!
          * \brief Indicates the minimum device password length that user can enter.
          *
          * This setting indicates the minimum device password length that user can enter.
          * Value - Integer. The minimum device password length.
          */
        MinDevicePasswordLength = 1,

        /*!
          * \brief Indicates whether the ability to use a simple password.
          *
          * This setting enables or disables the ability to use a simple password such as 1234
          * (numbers 0-9 only).
          * Value - Boolean. True if simple passwords are allowed, false otherwise.
          */
        AllowSimpleDevicePassword = 2,

        /*!
          * \brief Indicates whether a device password requires to be an alphanumeric.
          *
          * This setting indicates whether a device password requires to contain numeric and
          * non-numeric characters.
          * Value - Boolean. True if password has to be an alphanumeric, false otherwise
          */
        AlphaNumericDevicePasswordRequired = 3,

        /*!
          * \brief Indicates the number of complex characters that the password must contain.
          *
          * This setting indicates the number of complex characters (numbers and symbols,
          * not letters) that the password must contain.
          * Value - Integer. The number of complex characters that the password must contain.
          */
        MinDevicePasswordComplexCharacters = 4,

        // Device lock specific requirements
        /*!
          * \brief Indicates the number of seconds of inactivity before the device locks itself.
          *
          * This setting indicates the number of seconds of inactivity before the device locks itself.
          * Value - Integer. Number of seconds of inactivity before the device locks itself.
          */
        MaxInactivityTimeDeviceLock = 5,

        /*!
          * \brief Indicates the number of password failures that are permitted before the device is wiped.
          *
          * This setting indicates the number of password failures that are permitted before
          * the device is wiped.
          * Value - Unsigned integer (32bits). The number of password failures that are permitted
          * before the device is wiped.
          */
        MaxDevicePasswordFailedAttempts = 6,

        /*!
          * \brief Indicates whether the device password expires.
          *
          * This setting indicates whether the device password expires.
          * Value - Unsigned integer (32 bits). UINT_MAX for never to expire, < UINT_MAX actual to specify
          * number of days the password will be valid for.
          */
        DevicePasswordExpiration = 7,

        /*!
          * \brief Indicates whether the device stores previously used passwords.
          * This setting indicates whether the device stores previously used passwords.
          * Value - Integer. 0 if no history is to be stored, > 0 is the number of passwords to store in history
          */
        DevicePasswordHistory = 8,

        /*!
          * \brief Determine if the devicelock performs secure wipe or quick wipe of the device.
          * This setting indicates the level of security that is attached to wiping the device mmc.
          * Value - UInt 32. Use a value from Devicelock::DeviceLockEnums::WipeType
          */
        EraseDeviceType = 9,

        /*!
          * \brief Determine if a password change cancelling is not allowed
          *
          * Value - Boolean. Forbid the user to cancel the password change or not.
          */
        DevicePasswordForceChange = 10,

        /*!
          * \brief Determine the minimal allowed device wipe type
          *
          * Value - Integer
          * \sa WipeType
          */
        MinimalDeviceWipeTypeRequired = 11,

        /*!
          * \brief Allow the devicelock to interact with remote security features, for lock and wipe.
          *
          * Value - Boolean. true to allow, false to disable.
          */
        AllowRemoteSecurity = 12,

        /*!
          * \brief Set the device password if it is missing now.
          *
          * Value - String.
          */
        SetDevicePassword = 13,

        /*!
          * \brief Set a requirement to force an accepting of the entity.
          *
          * Value - Boolean. True if required.
          */
        ForceAccept = 14,

        /*!
          * \brief Header label to be shown on the change password request (if needed)
          *
          * Value - string
          */
        SenderHeaderLabel = 15,

        /*!
          * \brief Indicates that the type is unknown or does not exist
          */
        Unknown
    };

    /*!
      * \brief Register the enumerations with the QtMetaSystem and the QDBus Types
      * This method must be called before any class using these enumerations tries to use them in a connect statement.
      */
    static inline void registerLockEnumerations()
    {
        qRegisterMetaType<DeviceLockEnums::LockType>("DeviceLock::DeviceLockEnums::LockType");
        qRegisterMetaType<DeviceLockEnums::LockState>("DeviceLock::DeviceLockEnums::LockState");
        qRegisterMetaType<DeviceLockEnums::WipeType>("DeviceLock::DeviceLockEnums::WipeType");
        qRegisterMetaType<DeviceLockEnums::ProvisioningSettings>("DeviceLock::DeviceLockEnums::ProvisioningSettingsKey");
        qRegisterMetaType<DeviceLockEnums::UiPageType>("DeviceLock::DeviceLockEnums::UiPageType");
    }
};

//! Provisioning settings map - for DBus interface
typedef QMap<DeviceLockEnums::ProvisioningSettings, QVariant> ProvisioningSettingsMap;

//! Serialization for \a DeviceLockEnums::ProvisioningSettings
inline QDataStream &operator<<(QDataStream& stream,
                               const DeviceLockEnums::ProvisioningSettings &key)
{
    stream << static_cast<int>(key);
    return stream;
}

//! Deserialization for \a DeviceLockEnums::ProvisioningSettings
inline QDataStream &operator>>(QDataStream& stream, DeviceLockEnums::ProvisioningSettings &key)
{
    int value;
    stream >> value;
    key = static_cast<DeviceLockEnums::ProvisioningSettings>(value);
    return stream;
}

/*!
  * \brief A Method to serialize the ProvisioningSettingsMap over D-Bus
  * \param argument The data that will be sent over DBus
  * \param settings The Provisional settings that are to be sent
  */
inline QDBusArgument &operator<<(QDBusArgument &argument, const DeviceLock::ProvisioningSettingsMap &settings)
{
    argument.beginMap(QVariant::Int, qMetaTypeId<QDBusVariant>());
    DeviceLock::ProvisioningSettingsMap::const_iterator it = settings.constBegin();
    DeviceLock::ProvisioningSettingsMap::const_iterator end = settings.constEnd();
    for(; it != end; ++it)
    {
        argument.beginMapEntry();
        argument << static_cast<int>(it.key()) << QDBusVariant(it.value());
        argument.endMapEntry();
    }
    argument.endMap();
    return argument;
}

/*!
  * \brief A Method to de-serialize the ProvisioningSettingsMap from D-Bus
  * \param argument The data that will has been received from DBus
  * \param settings The Provisional settings container that will store the settings received from D-Bus
  */
inline const QDBusArgument &operator>>(const QDBusArgument &argument, DeviceLock::ProvisioningSettingsMap &settings)
{
    argument.beginMap();
    settings.clear();

    while (!argument.atEnd())
    {
        int key;
        QDBusVariant value;

        argument.beginMapEntry();
        argument >> key >> value;
        argument.endMapEntry();

        settings.insertMulti(static_cast<DeviceLock::DeviceLockEnums::ProvisioningSettings>(key),
                             value.variant());
    }

    argument.endMap();
    return argument;
}

} // DeviceLock

inline QDBusArgument &operator<<(QDBusArgument &argument, const QVariantHash &settings)
{
    argument.beginMap(QVariant::String, qMetaTypeId<QDBusVariant>());
    QVariantHash::ConstIterator it = settings.constBegin();
    QVariantHash::ConstIterator end = settings.constEnd();
    for(; it != end; ++it)
    {
        argument.beginMapEntry();
        argument << it.key() << QDBusVariant(it.value());
        argument.endMapEntry();
    }
    argument.endMap();
    return argument;
}

Q_DECLARE_METATYPE(DeviceLock::DeviceLockEnums::LockType);
Q_DECLARE_METATYPE(DeviceLock::DeviceLockEnums::LockState);
Q_DECLARE_METATYPE(DeviceLock::DeviceLockEnums::ProvisioningSettings);
Q_DECLARE_METATYPE(DeviceLock::ProvisioningSettingsMap);
Q_DECLARE_METATYPE(DeviceLock::DeviceLockEnums::UiPageType);

#endif // define QT_CORE_LIB

#endif // DEVICELOCK_H
