/*
  * Copyright (C) 2006-2011 Nokia Corporation.
  *
  * This file is part of devicelock
  *
  * Authors: Brian McGillion <brian.mcgillion@symbio.com>,
  * 	     Denis Mingulov  <denis.mingulov@symbio.com>
  *
  */

/*!
  * \file devicelockif.h
  * \class DeviceLockIf
  * \brief Implementation of a mechanism to allow the controlling devicelock over DBus.
  * \credential DeviceLockControl
  */

#ifndef DEVICELOCKIF_H
#define DEVICELOCKIF_H

#include <QtCore/QObject>
#include <QtCore/QByteArray>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QStringList>
#include <QtCore/QVariant>
#include <QtDBus/QtDBus>
#include <devicelock.h>

/*
 * Proxy class for interface com.nokia.devicelock
 */
#include "libdevicelock_global.h"

namespace DeviceLock
{

class LIBDEVICELOCKSHARED_EXPORT DeviceLockIf: public QDBusAbstractInterface
{
    Q_OBJECT
public:
    /*!
      * \brief The interface name of the connection
      * \returns The interface name.
      */
    static inline const char *staticInterfaceName()
    { return "com.nokia.devicelock"; }

public:

    /*!
      * \brief Constructor
      * \param parent The parent object of this.
      */
    DeviceLockIf(QObject *parent = 0);

    /*!
      * \brief Destructor
      */
    ~DeviceLockIf();

public Q_SLOTS: // METHODS

    /*!
      * \brief A plugin or client has requested to change some devicelock settings.
      * \param settingName The name of the settings to change
      * \param value The new value of the setting.
      * \param winId Windows Id, might be needed for chaining
      * \returns true if the change setting request is being processed, false otherwise.
      */
    inline QDBusPendingReply<bool> changeSetting(const QString &settingName, const QDBusVariant &value, uint winId = 0)
    {
        QList<QVariant> argumentList;
        argumentList << qVariantFromValue(settingName) << qVariantFromValue(value) << qVariantFromValue(winId);
        return asyncCallWithArgumentList(QLatin1String("changeSetting"), argumentList);
    }

    /*!
      * \brief Execute the data wipe
      * \param type The type of data wipe which has to be started
      * \returns true if data wipe started successfully, false otherwise
      * \credential State_WipeMMC
      */
    inline QDBusPendingReply<bool> dataWipe(int type)
    {
        QList<QVariant> argumentList;
        argumentList << qVariantFromValue(type);
        return asyncCallWithArgumentList(QLatin1String("dataWipe"), argumentList);
    }

    /*!
      * \brief Inquire after the state of a particular lock
      * \param lockType The type of the lock that state information is required for
      * \returns The state of that lock if it exists, DeviceLock::Undefined if the lock does not exist
      * \sa DeviceLock::DeviceLockEnums::LockState
      * \sa DeviceLock::DeviceLockEnums::LockType
      */
    inline QDBusPendingReply<int> getState(int lockType)
    {
        QList<QVariant> argumentList;
        argumentList << qVariantFromValue(lockType);
        return asyncCallWithArgumentList(QLatin1String("getState"), argumentList);
    }

    /*!
       \brief Remove the provisioning settings from the entity
       \param entity The entity which has to be removed
       \returns true if the request to remove was queued successfully, false otherwise
       \sa updateProvisioningSettingsResult
     */
    inline QDBusPendingReply<bool> removeProvisioningSettings(const QString &entity)
    {
        QList<QVariant> argumentList;
        argumentList << qVariantFromValue(entity);
        return asyncCallWithArgumentList(QLatin1String("removeProvisioningSettings"), argumentList);
    }

    /*!
      * \brief allow an external entity to set a password, if one has not already been configured.
      * \param password The password to replace the current one with.
      * \param forceChange Force the user to change this password
      * \returns true if the password was set, false if there already existed a password.
      * \credential DeviceLock_SetPassword
      */
    inline QDBusPendingReply<bool> setPassword(const QByteArray &password, bool forceChange)
    {
        QList<QVariant> argumentList;
        argumentList << qVariantFromValue(password) << qVariantFromValue(forceChange);
        return asyncCallWithArgumentList(QLatin1String("setPassword"), argumentList);
    }

    /*!
      * \brief Set the state of a particular lock, as defined by lockType to the state defined by state
      * \param lockType The type of lock for which the state is to be changed
      * \param state The state that the lock is to be set into
      * \returns true If the state was changed successfully, false otherwise
      * \credential State_Inhibit When inhibiting is required
      * \credential State_Locked When locking is required
      * \credential State_Unlocked When unlocking is required
      */
    inline QDBusPendingReply<bool> setState(int lockType, int state)
    {
        QList<QVariant> argumentList;
        argumentList << qVariantFromValue(lockType) << qVariantFromValue(state);
        return asyncCallWithArgumentList(QLatin1String("setState"), argumentList);
    }

    /*!
       \brief Set up or update the provisioning settings from the entity
       \param entity The entity which wants to update its provisioning settings
       \param settings The provisioning settings which have to be applied
       \returns true if the request to update was queued successfully, false otherwise
       \sa updateProvisioningSettingsResult
     */
    inline QDBusPendingReply<bool> updateProvisioningSettings(const QString &entity, const DeviceLock::ProvisioningSettingsMap & settings)
    {
        QList<QVariant> argumentList;
        argumentList << qVariantFromValue(entity) << qVariantFromValue(settings);
        return asyncCallWithArgumentList(QLatin1String("updateProvisioningSettings"), argumentList);
    }

Q_SIGNALS: // SIGNALS

    /*!
      * \brief Change the deviceLock password that is stored in the SingleSignon service
      * \param lockCode The new password that is going to be stored
      * \param oldPassword The oldPassword that is to be changed
      * Both passwords should be first encrypted before this method is called.  There is a shared
      * token that will be available, that can be used.
      */
    void changePassword(const QByteArray &lockCode, const QByteArray &oldPassword);

    /*!
      * \brief Return the result of a password prompt request.
      * \param result The result of the password prompt.  true on success, false otherwise.
      */
    void passwordPromptResult(bool result);

    /*!
      * \brief Notify a listner that the settings have changed
      * \param settings The new settings that have been configured.
      */
    void settingsChanged(const QVariantHash & settings);

    /*!
      * \brief Notify a listner that the settings have changed
      * \param settings The new compressed settings that have been configured.
      */
    void settingsChanged(const QByteArray &settings);

    /*!
      * \brief A signal to notify a dbus listner that the state of a lock has changed
      * \param lockType The lock that has changed
      * \param state The new state that the lock is is
      */
    void stateChanged(int lockType, int state);

    /*!
      * \brief A Signal that is emitted after active sync configuration has been requested.
      * \param entity The entity which wants to update its provisioning settings
      * \param result true if the provisioning settings were applied, false otherwise
      */
    void updateProvisioningSettingsResult(const QString &entity, bool settings);

private slots:
    /*!
      * \brief Handle internal settingsChanged signal - and emit public one
      * \param settings The new compressed settings that have been configured.
      */
    void handleSettingsChanged(const QByteArray &settings);
};

namespace com {
  namespace nokia {
    typedef DeviceLock::DeviceLockIf devicelock;
  }
}
}
#endif
