/*
  * Copyright (C) 2006-2011 Nokia Corporation.
  *
  * This file is part of devicelock
  *
  * Authors: Brian McGillion <brian.mcgillion@symbio.com>,
  * 	     Denis Mingulov  <denis.mingulov@symbio.com>
  *
  */

/*!
  * \file devicelockuiplugininterface.h
  * \author Brian McGillion <brian.mcgillion@symbio.com>
  * \class DeviceLockUIPluginInterface
  * \brief This class provides a plugin extension system that allows for a lock UI to be loaded.
  * This lock UI will be the primary interface that the users will see when the device is locked and
  * they are required to present their credentials.
  */

#ifndef DEVICELOCKUIPLUGININTERFACE_H
#define DEVICELOCKUIPLUGININTERFACE_H

#include <devicelock.h>
#include <lockuisignals.h>

#include <QtCore/QObject>
#include <QtCore/QString>
#include <QtCore/QtPlugin>
#include <QtCore/QVariant>

class DeviceLockUIPluginInterface : public QObject
{

public:
    /*!
      * \brief destructor
      */
    virtual ~DeviceLockUIPluginInterface() {}

    /*!
      * \brief Initialize the plugin
      * \param signalClass Signals that are shared between the core and the plugin.
      * \returns true If the initalization suceeded of false otherwise
      * This method is to allow the plugin to perform all tasks that it requires in order to present
      * the user with the locked screen
      * NB *** This method should not initialize any windows or widgets, it should just connect
      * the signals from LockUiSignals to internal signals in the plugin.  The thread behaviour of
      * the application will be undefined if any gui related initialization is performed.
      */
    virtual bool initialize(LockuiSignals *signalClass) = 0;

    /*!
      * \brief Allow the plugin notification that it is to clean up it's resources.
      * This will happen before the plugin is unloaded.
      */
    virtual void finalize() = 0;

    /*!
      * \brief Create a page to display to the user, e.g. a lock ui or a password prompt, change password
      * \param pageType The type of page to display to the user.
      * \param settings The public settings that can be shared with the UI.
      */
    virtual void createPage(DeviceLock::DeviceLockEnums::UiPageType pageType, const QVariantHash &settings) = 0;

    /*!
      * \brief Show a notification to display to the user based on the provided \a setting
      * \param setting The type of the notification
      */
    virtual void showNotification(const QString &setting) = 0;

public slots:
    /*!
      * \brief Receive updated settings from the core.
      * \param settings The updated settings.
      */
    virtual void updateSettings(const QVariantHash &settings) = 0;
};

//! Export a uniquely registered interface
Q_DECLARE_INTERFACE(DeviceLockUIPluginInterface, "com.nokia.devicelock.lockui.PluginInterface/1.0");

#endif // DEVICELOCKUIPLUGININTERFACE_H
